@@ -74,7 +74,8 @@ class ContractAssignmentInline(admin.TabularInline):
     raw_id_fields = ('user',)
 
     def queryset(self, request):
-        return super(ContractAssignmentInline, self).queryset(request).select_related()
+        qs = super(ContractAssignmentInline, self).queryset(request)
+        return qs.select_related()
 
 
 class ContractMilestoneInline(admin.TabularInline):
@@ -175,18 +176,6 @@ class PersonScheduleAdmin(admin.ModelAdmin):
 admin.site.register(timepiece.PersonSchedule, PersonScheduleAdmin)
 
 
-class RepeatPeriodAdmin(admin.ModelAdmin):
-    list_display = ('count', 'interval')
-    list_filter = ('interval',)
-admin.site.register(timepiece.RepeatPeriod, RepeatPeriodAdmin)
-
-
-class BillingWindowAdmin(admin.ModelAdmin):
-    list_display = ('id', 'period', 'date', 'end_date')
-    list_filter = ('period',)
-admin.site.register(timepiece.BillingWindow, BillingWindowAdmin)
-
-
 class LocationAdmin(admin.ModelAdmin):
     list_display = ('id', 'name')
 admin.site.register(timepiece.Location, LocationAdmin)
