@@ -13,15 +13,12 @@ from django.utils.translation import ugettext_lazy as _
 from django.core.exceptions import ValidationError, NON_FIELD_ERRORS
 
 from selectable import forms as selectable_forms
-from timepiece.lookups import ProjectLookup
 
-from ajax_select.fields import AutoCompleteSelectMultipleField, \
-                               AutoCompleteSelectField, \
-                               AutoCompleteSelectWidget
+from timepiece.lookups import ProjectLookup, QuickLookup
+from timepiece.lookups import UserLookup, BusinessLookup
 
-from timepiece.models import Project, Entry, Activity, UserProfile
-from timepiece.fields import PendulumDateTimeField
-from timepiece.widgets import PendulumDateTimeWidget, SecondsToHoursWidget
+from timepiece.models import Project, Entry, Activity, UserProfile, Attribute
+from timepiece.fields import UserModelChoiceField
 from timepiece import models as timepiece
 from timepiece import utils
 
@@ -69,7 +66,8 @@ class CreatePersonForm(auth_forms.UserCreationForm):
         model = auth_models.User
         fields = (
             "username", "first_name", "last_name",
-            "email", "is_active", "is_staff")
+            "email", "is_active", "is_staff"
+        )
 
 
 class EditPersonForm(auth_forms.UserChangeForm):
@@ -86,6 +84,9 @@ class EditPersonForm(auth_forms.UserChangeForm):
             "email", "is_active", "is_staff"
         )
 
+    def clean_password(self):
+        return self.cleaned_data.get('password_one', None)
+
     def clean(self):
         super(EditPersonForm, self).clean()
         password_one = self.cleaned_data.get('password_one', None)
@@ -106,55 +107,70 @@ class EditPersonForm(auth_forms.UserChangeForm):
         return instance
 
 
-class CharAutoCompleteSelectWidget(AutoCompleteSelectWidget):
-    def value_from_datadict(self, data, files, name):
-        return data.get(name, None)
-
-
 class QuickSearchForm(forms.Form):
-    quick_search = AutoCompleteSelectField(
-        'quick_search',
-        widget=CharAutoCompleteSelectWidget('quick_search'),
+    quick_search = selectable_forms.AutoCompleteSelectField(
+        QuickLookup,
+        label='Quick Search',
+        required=False
     )
+    quick_search.widget.attrs['placeholder'] = 'Search'
 
     def clean_quick_search(self):
         item = self.cleaned_data['quick_search']
-        if isinstance(item, timepiece.Project):
-            return reverse('view_project', kwargs={
-                'project_id': item.id,
-            })
-        elif isinstance(item, timepiece.Business,):
-            return reverse('view_business', kwargs={
-                'business': item.id,
-            })
-        elif isinstance(item, auth_models.User,):
-            return reverse('view_person', kwargs={
-                'person_id': item.id,
-            })
-        raise forms.ValidationError('Must be a User or Project')
+
+        if item is not None:
+            try:
+                item = item.split('-')
+                if len(item) == 1 or '' in item:
+                    raise ValueError
+                return item
+            except ValueError:
+                raise forms.ValidationError('%s' %
+                    'User, business, or project does not exist')
+        else:
+            raise forms.ValidationError('%s' %
+                'User, business, or project does not exist')
 
     def save(self):
-        return self.cleaned_data['quick_search']
+        type, pk = self.cleaned_data['quick_search']
 
+        if type == 'individual':
+            return reverse('view_person', kwargs={
+                'person_id': pk
+            })
+        elif type == 'business':
+            return reverse('view_business', kwargs={
+                'business': pk
+            })
+        elif type == 'project':
+            return reverse('view_project', kwargs={
+                'project_id': pk
+            })
 
-class SearchForm(forms.Form):
-    search = forms.CharField(required=False)
+        raise forms.ValidationError('Must be a user, project, or business')
 
 
 class AddUserToProjectForm(forms.Form):
-    user = AutoCompleteSelectField('user')
+    user = selectable_forms.AutoCompleteSelectField(UserLookup, label="")
+    user.widget.attrs['placeholder'] = 'Add User'
 
     def save(self):
         return self.cleaned_data['user']
 
 
 class ClockInForm(forms.ModelForm):
+    active_comment = forms.CharField(label='Notes for the current entry',
+                                     widget=forms.Textarea, required=False)
+
     class Meta:
         model = timepiece.Entry
-        fields = ('location', 'project', 'activity', 'start_time',)
+        fields = (
+            'active_comment', 'location', 'project', 'activity', 'start_time',
+        )
 
     def __init__(self, *args, **kwargs):
         self.user = kwargs.pop('user')
+        self.active = kwargs.pop('active', None)
         initial = kwargs.get('initial', {})
         default_loc = getattr(
             settings,
@@ -187,13 +203,19 @@ class ClockInForm(forms.ModelForm):
             users=self.user, status__enable_timetracking=True,
             type__enable_timetracking=True
         )
+        if not self.active:
+            self.fields.pop('active_comment')
+        else:
+            self.fields['active_comment'].initial = self.active.comments
         self.instance.user = self.user
 
     def clean_start_time(self):
         """
         Make sure that the start time doesn't come before the active entry
         """
-        start = self.cleaned_data['start_time']
+        start = self.cleaned_data.get('start_time')
+        if not start:
+            return start
         active_entries = self.user.timepiece_entries.filter(
             start_time__gte=start, end_time__isnull=True)
         for entry in active_entries:
@@ -203,12 +225,27 @@ class ClockInForm(forms.ModelForm):
             raise forms.ValidationError(output)
         return start
 
+    def clean(self):
+        start_time = self.clean_start_time()
+        data = self.cleaned_data
+        if not start_time:
+            return data
+        if self.active:
+            self.active.unpause()
+            self.active.comments = data['active_comment']
+            self.active.end_time = start_time - timedelta(seconds=1)
+            if not self.active.clean():
+                raise forms.ValidationError(data)
+        return data
+
     def save(self, commit=True):
         entry = super(ClockInForm, self).save(commit=False)
         entry.hours = 0
         entry.clock_in(self.user, self.cleaned_data['project'])
         if commit:
             entry.save()
+            if self.active:
+                self.active.save()
         return entry
 
 
@@ -274,7 +311,8 @@ class AddUpdateEntryForm(forms.ModelForm):
         self.user = kwargs.pop('user')
         super(AddUpdateEntryForm, self).__init__(*args, **kwargs)
         self.fields['project'].queryset = timepiece.Project.objects.filter(
-            users=self.user,
+            users=self.user, status__enable_timetracking=True,
+            type__enable_timetracking=True
         )
         #if editing a current entry, remove the end time field
         if self.instance.start_time and not self.instance.end_time:
@@ -384,9 +422,20 @@ class YearMonthForm(forms.Form):
         year = int(self.cleaned_data.get('year', this_year))
         from_date = datetime(year, month, 1)
         to_date = from_date + relativedelta(months=1)
+
         return (from_date, to_date)
 
 
+class UserYearMonthForm(YearMonthForm):
+    users = auth_models.User.objects.exclude(timepiece_entries=None) \
+        .order_by('first_name')
+    user = UserModelChoiceField(label='', queryset=users, required=False)
+
+    def save(self):
+        from_date, to_date = super(UserYearMonthForm, self).save()
+        return  (from_date, to_date, self.cleaned_data.get('user', None))
+
+
 class ProjectionForm(DateForm):
     user = forms.ModelChoiceField(queryset=None)
 
@@ -408,7 +457,7 @@ class ProjectForm(forms.ModelForm):
         fields = (
             'name',
             'business',
-            'trac_environment',
+            'tracker_url',
             'point_person',
             'type',
             'status',
@@ -416,6 +465,13 @@ class ProjectForm(forms.ModelForm):
             'description',
         )
 
+    business = selectable_forms.AutoCompleteSelectField(
+        BusinessLookup,
+        label='Business',
+        required=True
+    )
+    business.widget.attrs['placeholder'] = 'Search'
+
     def __init__(self, *args, **kwargs):
         super(ProjectForm, self).__init__(*args, **kwargs)
 
@@ -456,7 +512,8 @@ class InvoiceForm(forms.ModelForm):
 
 
 class SearchForm(forms.Form):
-    search = forms.CharField(required=False)
+    search = forms.CharField(required=False, label='')
+    search.widget.attrs['placeholder'] = 'Search'
 
 
 class UserForm(forms.ModelForm):
@@ -476,3 +533,40 @@ class UserProfileForm(forms.ModelForm):
     class Meta:
         model = timepiece.UserProfile
         exclude = ('user',)
+
+
+class ProjectSearchForm(forms.Form):
+    search = forms.CharField(required=False, label='')
+    search.widget.attrs['placeholder'] = 'Search'
+    status = forms.ChoiceField(required=False, choices=[], label='')
+
+    def __init__(self, *args, **kwargs):
+        super(ProjectSearchForm, self).__init__(*args, **kwargs)
+        PROJ_STATUS_CHOICES = [('', 'Any Status')]
+        PROJ_STATUS_CHOICES.extend([(a.pk, a.label) for a
+                in Attribute.objects.all().filter(type="project-status")])
+        self.fields['status'].choices = PROJ_STATUS_CHOICES
+
+    def save(self):
+        search = self.cleaned_data.get('search', '')
+        status = self.cleaned_data.get('status', '')
+        return (search, status)
+
+
+class DeleteForm(forms.Form):
+    """
+    Returns True if the object was deleted
+    """
+    def __init__(self, *args, **kwargs):
+        self.instance = kwargs.pop('instance', None)
+        super(DeleteForm, self).__init__(*args, **kwargs)
+
+    def save(self):
+        if self.instance:
+            try:
+                self.instance.delete()
+            except AssertionError:
+                return False
+            else:
+                return True
+        return False
