@@ -68,7 +68,8 @@ class Business(models.Model):
 
 class Project(models.Model):
     name = models.CharField(max_length=255)
-    trac_environment = models.CharField(max_length=255, blank=True, null=True)
+    tracker_url = models.CharField(max_length=255, blank=True, null=False,
+        default="")
     business = models.ForeignKey(
         Business,
         related_name='new_business_projects',
@@ -105,25 +106,26 @@ class Project(models.Model):
             ('email_project_report', 'Can email project report'),
             ('view_project_time_sheet', 'Can view project time sheet'),
             ('export_project_time_sheet', 'Can export project time sheet'),
+            ('generate_project_invoice', 'Can generate project invoice'),
         )
 
     def __unicode__(self):
         return self.name
 
     def trac_url(self):
-        return settings.TRAC_URL % self.trac_environment
+        return settings.TRAC_URL % self.tracker_url
 
 
 class RelationshipType(models.Model):
     name = models.CharField(max_length=255, unique=True)
     slug = models.CharField(max_length=255, unique=True, editable=False)
 
-    def save(self):
+    def save(self, *args, **kwargs):
         queryset = RelationshipType.objects.all()
         if self.id:
             queryset = queryset.exclude(id__exact=self.id)
         self.slug = utils.slugify_uniquely(self.name, queryset, 'slug')
-        super(RelationshipType, self).save()
+        super(RelationshipType, self).save(*args, **kwargs)
 
     def __unicode__(self):
         return self.name
@@ -507,11 +509,24 @@ class Entry(models.Model):
             pass
         if end <= start:
             raise ValidationError('Ending time must exceed the starting time')
+        delta = (end - start)
+        delta_secs = (delta.seconds + delta.days * 24 * 60 * 60)
+        limit_secs = 60 * 60 * 12
+        if delta_secs > limit_secs or self.seconds_paused > limit_secs:
+            err_msg = 'Ending time exceeds starting time by 12 hours or more '\
+                'for {0} on {1} at {2} to {3} at {4}.'.format(
+                    self.project.name,
+                    start.strftime('%m/%d/%Y'),
+                    start.strftime('%H:%M:%S'),
+                    end.strftime('%m/%d/%Y'),
+                    end.strftime('%H:%M:%S')
+                )
+            raise ValidationError(err_msg)
         return True
 
-    def save(self, **kwargs):
+    def save(self, *args, **kwargs):
         self.hours = Decimal('%.2f' % round(self.total_hours, 2))
-        super(Entry, self).save(**kwargs)
+        super(Entry, self).save(*args, **kwargs)
 
     def get_seconds(self):
         """
@@ -675,6 +690,7 @@ class Entry(models.Model):
             ('can_pause', 'Can pause and unpause log entries'),
             ('can_clock_out', 'Can use Pendulum to clock out'),
             ('view_entry_summary', 'Can view entry summary page'),
+            ('view_payroll_summary', 'Can view payroll summary page'),
         )
 
 
