@@ -12,7 +12,6 @@ from django.core.urlresolvers import reverse
 from dateutil.relativedelta import relativedelta
 from dateutil import rrule
 
-from timepiece.models import PersonRepeatPeriod, AssignmentAllocation
 import timepiece.models as timepiece
 from timepiece.utils import get_total_time, get_week_start, get_month_start
 
@@ -51,21 +50,6 @@ def bar_graph(context, name, worked, total, width=None, suffix=None):
         }
 
 
-@register.inclusion_tag('timepiece/time-sheet/my_ledger.html',
-                        takes_context=True)
-def my_ledger(context):
-    try:
-        period = PersonRepeatPeriod.objects.select_related(
-            'user',
-            'repeat_period',
-        ).get(
-            user=context['request'].user
-        )
-    except PersonRepeatPeriod.DoesNotExist:
-        return {'period': False}
-    return {'period': period}
-
-
 @register.inclusion_tag('timepiece/time-sheet/_date_filters.html',
     takes_context=True)
 def date_filters(context, options=None):
@@ -145,6 +129,7 @@ def invoice_subheaders(context, current):
         'invoice': context['invoice'],
     }
 
+
 @register.simple_tag
 def hours_for_assignment(assignment, date):
     end = date + relativedelta(days=5)
@@ -167,7 +152,8 @@ def total_allocated(assignment):
 @register.simple_tag
 def hours_for_week(user, date):
     end = date + relativedelta(days=5)
-    blocks = AssignmentAllocation.objects.filter(assignment__user=user,
+    blocks = timepiece.AssignmentAllocation.objects.filter(
+                                                 assignment__user=user,
                                                  date__gte=date, date__lte=end)
     hours = blocks.aggregate(hours=Sum('hours'))['hours']
     if not hours:
