@@ -6,3 +6,8 @@ from timepiece.tests.invoices import *
 from timepiece.tests.user_settings import *
 from timepiece.tests.management import *
 from timepiece.tests.reports import *
+from timepiece.tests.search import *
+from timepiece.tests.project_list import *
+from timepiece.tests.general_ledger import *
+from timepiece.tests.crmcontent import *
+from timepiece.tests.dashboard import *
