@@ -3,6 +3,7 @@ from decimal import Decimal
 
 from django.conf import settings
 from django.core.urlresolvers import reverse
+from django.contrib.auth.models import Permission
 
 from timepiece import models as timepiece
 from timepiece import forms as timepiece_forms
@@ -151,6 +152,7 @@ class PayrollTest(TimepieceDataTestCase):
         self.client.login(username='superuser', password='abc')
         response = self.client.get(self.url, self.args)
         monthly_totals = response.context['monthly_totals']
+        # Test the first entry
         self.assertEqual(monthly_totals[0][1],
                          [(Decimal('45.00'), 81.82),
                           (Decimal('10.00'), 18.18),
@@ -160,3 +162,42 @@ class PayrollTest(TimepieceDataTestCase):
                          [(u'vacation', Decimal('20.00')),
                           (u'sick', Decimal('40.00'))])
         self.assertEqual(monthly_totals[0][3], Decimal('115.00'))
+        # Test the totals row at the bottom.
+        self.assertEqual(monthly_totals[-1][1],
+                         [(Decimal('90.00'), 81.82),
+                          (Decimal('20.00'), 18.18),
+                          Decimal('110.00')
+                         ])
+        self.assertEqual(monthly_totals[-1][2],
+                         [(u'vacation', Decimal('40.00')),
+                          (u'sick', Decimal('80.00'))])
+        self.assertEqual(monthly_totals[-1][3], Decimal('230.00'))
+
+    def testNoPermission(self):
+        """
+        Regular users shouldn't be able to retrieve the payroll report
+        page.
+
+        """
+        self.client.login(username='user', password='abc')
+        response = self.client.get(self.url, self.args)
+        self.assertEqual(response.status_code, 302)
+
+    def testSuperUserPermission(self):
+        """Super users should be able to retrieve the payroll report page."""
+        self.client.login(username='superuser', password='abc')
+        response = self.client.get(self.url, self.args)
+        self.assertEqual(response.status_code, 200)
+
+    def testPayrollPermission(self):
+        """
+        If a regular user is given the view_payroll_summary permission, they
+        should be able to retrieve the payroll summary page.
+
+        """
+        self.client.login(username='user', password='abc')
+        payroll_perm = Permission.objects.get(codename='view_payroll_summary')
+        self.user.user_permissions.add(payroll_perm)
+        self.user.save()
+        response = self.client.get(self.url, self.args)
+        self.assertEqual(response.status_code, 200)
