@@ -54,6 +54,29 @@ class MyLedgerTest(TimepieceDataTestCase):
                            kwargs={'user_id': self.user.pk}
         )
 
+    def login_with_permissions(self):
+        view_entry_summary = Permission.objects.get(
+            codename='view_entry_summary')
+        user = User.objects.create_user('perm', 'e@e.com', 'abc')
+        user.user_permissions.add(view_entry_summary)
+        user.save()
+
+        self.client.login(username='perm', password='abc')
+
+    def test_timesheet_view_permission(self):
+        """A user with the correct permissions should see the menu"""
+        self.login_with_permissions()
+        response = self.client.get(self.url)
+        self.assertEquals(response.status_code, 200)
+        self.assertTrue('user' in response.context['year_month_form'].fields)
+
+    def test_timesheet_view_no_permission(self):
+        """A regular user should not see the user menu"""
+        self.client.login(username='user', password='abc')
+        response = self.client.get(self.url)
+        self.assertTrue(response.status_code, 200)
+        self.assertFalse('user' in response.context['year_month_form'].fields)
+
     def testEmptyTimeSheet(self):
         self.client.login(username='user', password='abc')
         response = self.client.get(self.url)
@@ -150,9 +173,7 @@ class ClockInTest(TimepieceDataTestCase):
         }
 
     def testClockIn(self):
-        """
-        Test the simplest clock in scenario
-        """
+        """Test the simplest clock in scenario"""
         self.client.login(username='user', password='abc')
         data = self.clock_in_form
         response = self.client.post(self.url, data, follow=True)
@@ -190,6 +211,43 @@ class ClockInTest(TimepieceDataTestCase):
         self.assertEqual(closed_entry.end_time + datetime.timedelta(seconds=1),
                          current_entry.start_time)
 
+    def testClockInManyActive(self):
+        """
+        There should never be more than one active entry. If this happens,
+        there is not a clean way to auto-clock out. Redirect to dashboard.
+        """
+        self.client.login(username='user', password='abc')
+        entry1 = self.create_entry({
+            'start_time': self.ten_min_ago,
+        })
+        entry2 = self.create_entry({
+            'start_time': self.now - datetime.timedelta(minutes=20),
+        })
+        data = self.clock_in_form
+        data.update({
+            'start_time_0': self.now.strftime('%m/%d/%Y'),
+            'start_time_1': self.now.strftime('%H:%M:%S'),
+        })
+        response = self.client.post(self.url, data, follow=True)
+        self.assertRedirects(response, reverse('timepiece-entries'),
+                             status_code=302, target_status_code=200)
+        message = response.context['messages']._loaded_messages[0].message
+        self.assertTrue(message.startswith('You have more than one active'))
+
+    def testClockInCurrentStatus(self):
+        """Verify the status of the current entry shows what is expected"""
+        self.client.login(username='user', password='abc')
+        entry1 = self.create_entry({
+            'start_time': self.ten_min_ago,
+        })
+        data = self.clock_in_form
+        data.update({
+            'start_time_0': self.now.strftime('%m/%d/%Y'),
+            'start_time_1': self.now.strftime('%H:%M:%S'),
+        })
+        response = self.client.get(self.url, data)
+        self.assertEqual(response.context['active'], entry1)
+
     def testClockInPause(self):
         """
         Test that the user can clock in while the current entry is paused.
@@ -205,12 +263,14 @@ class ClockInTest(TimepieceDataTestCase):
         data.update({
             'start_time_0': self.now.strftime('%m/%d/%Y'),
             'start_time_1': self.now.strftime('%H:%M:%S'),
+            'active_comment': 'test comment',
         })
         response = self.client.post(self.url, data, follow=True)
         #obtain entry1 now that it is closed. The hours should be recorded
         e_id = timepiece.Entry.objects.get(pk=entry1.id)
         self.assertTrue(e_id.is_closed)
         self.assertTrue(e_id.hours)
+        self.assertEqual(e_id.comments, 'test comment')
 
     def testClockInBlock(self):
         """
@@ -299,6 +359,80 @@ class ClockInTest(TimepieceDataTestCase):
             'The start time is on or before the current entry: ' + \
             '%(project)s - %(activity)s starting at %(st_str)s' % entry1_data)
 
+    def testClockInActiveTooLong(self):
+        """
+        Test that if the active entry is too long, the clock in form will
+        invalidate
+        """
+        self.client.login(username='user', password='abc')
+        entry1 = self.create_entry({
+            'start_time': self.now - datetime.timedelta(hours=13),
+        })
+        end_time = self.now - datetime.timedelta(seconds=1)
+        data = self.clock_in_form
+        data.update({
+            'start_time_0': self.now.strftime('%m/%d/%Y'),
+            'start_time_1': self.now.strftime('%H:%M:%S'),
+        })
+        response = self.client.post(self.url, data)
+        err_msg = 'Ending time exceeds starting time by 12 hours ' \
+            'or more for {0} on {1} at {2} to {3} at {4}.'.format(
+                entry1.project.name,
+                entry1.start_time.strftime('%m/%d/%Y'),
+                entry1.start_time.strftime('%H:%M:%S'),
+                end_time.strftime('%m/%d/%Y'),
+                end_time.strftime('%H:%M:%S')
+            )
+        self.assertFormError(response, 'form', None, err_msg)
+
+    def test_clockin_error_active_entry(self):
+        """
+        If you have an active entry and clock in to another,
+        you should not be clocked out of the current active entry
+        if the clock in form contains errors
+        """
+        self.client.login(username='user', password='abc')
+
+        # Create a valid entry and follow the redirect to the homepage
+        response = self.client.post(self.url, self.clock_in_form, follow=True)
+        self.assertEquals(response.status_code, 200)
+        self.assertTrue(response.context['messages'])
+
+        data = self.clock_in_form
+        data.update({'start_time_0': None})
+        response = self.client.post(self.url, data)
+
+        msg = 'Enter a valid date/time.'
+        self.assertFormError(response, 'form', 'start_time', msg)
+
+        active = timepiece.Entry.objects.get()
+        self.assertIsNone(active.end_time)
+
+    def test_clockin_correct_active_entry(self):
+        """
+        If you clock in with an an active entry, that entry
+        should be clocked out
+        """
+        self.client.login(username='user', password='abc')
+
+        # Create a valid entry and follow the redirect to the homepage
+        response = self.client.post(self.url, self.clock_in_form, follow=True)
+        self.assertEquals(response.status_code, 200)
+        self.assertTrue(response.context['messages'])
+
+        active = timepiece.Entry.objects.get()
+
+        data = self.clock_in_form
+        start_time = self.now + datetime.timedelta(seconds=10)
+        data.update({
+            'start_time_0': start_time.strftime('%m/%d/%Y'),
+            'start_time_1': start_time.strftime('%H:%M:%S')
+        })
+        response = self.client.post(self.url, data)
+
+        active = timepiece.Entry.objects.get(pk=active.pk)
+        self.assertIsNotNone(active.end_time)
+
     def testProjectListFiltered(self):
         self.client.login(username='user', password='abc')
         response = self.client.get(self.url)
@@ -306,6 +440,11 @@ class ClockInTest(TimepieceDataTestCase):
         projects = list(response.context['form'].fields['project'].queryset)
         self.assertTrue(self.project in projects)
         self.assertFalse(self.project2 in projects)
+        self.project.status.enable_timetracking = False
+        self.project.status.save()
+        response = self.client.get(self.url)
+        projects = list(response.context['form'].fields['project'].queryset)
+        self.assertTrue(self.project not in projects)
 
     def testClockInLogin(self):
         response = self.client.get(self.url)
@@ -337,6 +476,22 @@ class ClockInTest(TimepieceDataTestCase):
         err_msg += 'choose among development, and Work'
         self.assertFormError(response, 'form', None, err_msg)
 
+    def test_clock_in_active_comments(self):
+        """
+        Comments left from editing the current active entry should appear
+        if you are clocking in
+        """
+        entry = self.create_entry({
+            'start_time': self.ten_min_ago
+        })
+        entry.comments = u'Some comments'
+        entry.save()
+
+        self.client.login(username='user', password='abc')
+
+        response = self.client.get(self.url)
+        self.assertContains(response, 'Some comments')
+
 
 class AutoActivityTest(TimepieceDataTestCase):
     """Test the initial value chosen for activity on clock in form"""
@@ -492,6 +647,49 @@ class ClockOutTest(TimepieceDataTestCase):
         self.assertFormError(response, 'form', None,
             'Ending time must exceed the starting time')
 
+    def testClockOutTooLong(self):
+        end_time = self.entry.start_time + datetime.timedelta(hours=13)
+        data = {
+            'start_time_0': self.entry.start_time.strftime('%m/%d/%Y'),
+            'start_time_1': self.entry.start_time.strftime('%H:%M:%S'),
+            'end_time_0': end_time.strftime('%m/%d/%Y'),
+            'end_time_1': end_time.strftime('%H:%M:%S'),
+            'location': self.location.pk,
+        }
+        response = self.client.post(self.url, data)
+        err_msg = 'Ending time exceeds starting time by 12 hours ' \
+            'or more for {0} on {1} at {2} to {3} at {4}.'.format(
+                self.entry.project.name,
+                self.entry.start_time.strftime('%m/%d/%Y'),
+                self.entry.start_time.strftime('%H:%M:%S'),
+                end_time.strftime('%m/%d/%Y'),
+                end_time.strftime('%H:%M:%S')
+            )
+        self.assertFormError(response, 'form', None, err_msg)
+
+    def testClockOutPauseTooLong(self):
+        paused_entry = self.entry
+        paused_entry.seconds_paused = 60 * 60 * 13
+        paused_entry.save()
+        data = {
+            'start_time_0': paused_entry.start_time.strftime('%m/%d/%Y'),
+            'start_time_1': paused_entry.start_time.strftime('%H:%M:%S'),
+            'end_time_0': self.default_end_time.strftime('%m/%d/%Y'),
+            'end_time_1': self.default_end_time.strftime('%H:%M:%S'),
+            'location': self.location.pk,
+        }
+        response = self.client.post(
+            reverse('timepiece-clock-out', args=[paused_entry.pk]), data)
+        err_msg = 'Ending time exceeds starting time by 12 hours ' \
+            'or more for {0} on {1} at {2} to {3} at {4}.'.format(
+                self.entry.project.name,
+                paused_entry.start_time.strftime('%m/%d/%Y'),
+                paused_entry.start_time.strftime('%H:%M:%S'),
+                self.default_end_time.strftime('%m/%d/%Y'),
+                self.default_end_time.strftime('%H:%M:%S')
+            )
+        self.assertFormError(response, 'form', None, err_msg)
+
     def testClockOutOverlap(self):
         """
         Test that the user cannot clock out if the times overlap with an
@@ -759,6 +957,37 @@ class CreateEditEntry(TimepieceDataTestCase):
             '%(project)s - %(activity)s starting at %(st_str)s' % \
             self.current_entry_data)
 
+    def testCreateTooLongEntry(self):
+        """
+        Test that the entry is blocked if the duration is too long.
+        """
+        long_entry = self.default_data
+        end_time = self.now + datetime.timedelta(hours=13)
+        long_entry.update({
+            'start_time_0': self.now.strftime('%m/%d/%Y'),
+            'start_time_1': self.now.strftime('%H:%M:%S'),
+            'end_time_0': end_time.strftime('%m/%d/%Y'),
+            'end_time_1': end_time.strftime('%H:%M:%S'),
+        })
+        response = self.client.post(self.create_url, long_entry, follow=True)
+        err_msg = 'Ending time exceeds starting time by 12 hours ' \
+            'or more for {0} on {1} at {2} to {3} at {4}.'.format(
+                self.project.name,
+                self.now.strftime('%m/%d/%Y'),
+                self.now.strftime('%H:%M:%S'),
+                end_time.strftime('%m/%d/%Y'),
+                end_time.strftime('%H:%M:%S')
+            )
+        self.assertFormError(response, 'form', None, err_msg)
+
+    def testCreateLongPauseEntry(self):
+        """
+        Test that the entry is blocked if the duration is too long.
+        """
+        long_pause = self.default_data
+        long_pause['seconds_paused'] = 60 * 60 * 13
+        response = self.client.post(self.create_url, long_pause, follow=True)
+
     def testProjectList(self):
         """
         Make sure the list of available projects conforms to user associations
@@ -767,7 +996,12 @@ class CreateEditEntry(TimepieceDataTestCase):
         self.assertEqual(response.status_code, 200)
         projects = list(response.context['form'].fields['project'].queryset)
         self.assertTrue(self.project in projects)
-        self.assertFalse(self.project2 in projects)
+        self.assertTrue(self.project2 not in projects)
+        self.project.status.enable_timetracking = False
+        self.project.status.save()
+        response = self.client.get(reverse('timepiece-add'))
+        projects = list(response.context['form'].fields['project'].queryset)
+        self.assertTrue(self.project not in projects)
 
     def testBadActivity(self):
         """
@@ -786,8 +1020,8 @@ class StatusTest(TimepieceDataTestCase):
     def setUp(self):
         super(StatusTest, self).setUp()
         self.client.login(username='user', password='abc')
-        now = datetime.datetime.now()
-        from_date = utils.get_month_start(now)
+        self.now = datetime.datetime.now()
+        from_date = utils.get_month_start(self.now)
         self.from_date = from_date
         self.sheet_url = reverse('view_person_time_sheet',
             args=[self.user.pk])
@@ -796,6 +1030,175 @@ class StatusTest(TimepieceDataTestCase):
         self.approve_url = reverse('change_person_time_sheet',
             args=['approve', self.user.pk, from_date.strftime("%Y-%m-%d")])
 
+    def get_reject_url(self, entry_id):
+        "Helper for the reject entry view"
+        return reverse('timepiece-reject-entry', args=[entry_id])
+
+    def login_as_admin(self):
+        "Helper to login as an admin user"
+        self.admin = User.objects.create_user('admin', 'e@e.com', 'abc')
+        self.admin.is_superuser = True
+        self.admin.save()
+        self.client.login(username='admin', password='abc')
+
+    def login_with_permission(self):
+        """Helper to login as a user with correct permissions"""
+        view_entry_summary = Permission.objects.get(
+            codename=('view_entry_summary'))
+        self.perm_user = User.objects.create_user('perm', 'e@e.com', 'abc')
+        self.perm_user.user_permissions.add(view_entry_summary)
+        self.perm_user.save()
+        self.client.login(username='perm', password='abc')
+
+    def test_verify_link(self):
+        entry = self.create_entry({
+            'user': self.user,
+            'start_time': self.now - datetime.timedelta(hours=1),
+            'end_time': self.now
+        })
+
+        response = self.client.get(self.sheet_url)
+        self.assertTrue(response.status_code, 200)
+
+        self.assertTrue(response.context['show_verify'])
+        self.assertFalse(response.context['show_approve'])
+
+    def test_approve_link(self):
+        entry = self.create_entry({
+            'user': self.user,
+            'start_time': self.now - datetime.timedelta(hours=1),
+            'end_time': self.now,
+            'status': 'verified'
+        })
+
+        response = self.client.get(self.sheet_url)
+        self.assertTrue(response.status_code)
+
+        self.assertTrue(response.context['show_approve'])
+        self.assertFalse(response.context['show_verify'])
+
+    def test_no_hours_verify(self):
+        response = self.client.get(self.verify_url, follow=True)
+        self.assertEquals(response.status_code, 200)
+
+        msg = 'You cannot verify/approve a timesheet with no hours'
+        messages = response.context['messages']
+        self.assertEquals(messages._loaded_messages[0].message, msg)
+
+        response = self.client.post(self.verify_url, follow=True)
+        self.assertEquals(messages._loaded_messages[0].message, msg)
+
+    def test_no_hours_approve(self):
+        self.login_with_permission()
+        response = self.client.get(self.approve_url, follow=True)
+        self.assertEquals(response.status_code, 200)
+
+        msg = 'You cannot verify/approve a timesheet with no hours'
+        messages = response.context['messages']
+        self.assertEquals(messages._loaded_messages[0].message, msg)
+
+        response = self.client.post(self.approve_url, follow=True)
+        self.assertEquals(messages._loaded_messages[0].message, msg)
+
+    def test_verify_other_user(self):
+        """A user should not be able to verify another's timesheet"""
+        entry = self.create_entry({
+            'user': self.user2,
+            'start_time': self.now - datetime.timedelta(hours=1),
+            'end_time': self.now
+        })
+
+        url = reverse('change_person_time_sheet',
+            args=('verify', self.user2.pk,
+                self.from_date.strftime('%Y-%m-%d')
+            )
+        )
+        response = self.client.get(url)
+
+        self.assertEquals(response.status_code, 403)
+        self.assertEquals(entry.status, 'unverified')
+
+        response = self.client.post(url, {'do_action': 'Yes'})
+        self.assertEquals(response.status_code, 403)
+        self.assertEquals(entry.status, 'unverified')
+
+    def test_approve_user(self):
+        """A regular user should not be able to approve their timesheet"""
+        entry = self.create_entry({
+            'user': self.user,
+            'start_time': self.now - datetime.timedelta(hours=1),
+            'end_time': self.now
+        })
+
+        response = self.client.get(self.approve_url)
+        self.assertEquals(response.status_code, 403)
+
+        response = self.client.post(self.approve_url, {'do_action': 'Yes'})
+        self.assertEquals(response.status_code, 403)
+        self.assertNotEquals(entry.status, 'approved')
+        self.assertContains(response,
+            'Forbidden: You cannot approve this timesheet',
+            status_code=403
+        )
+
+    def test_approve_other_user(self):
+        """A regular user should not be able to approve another's timesheet"""
+        entry = self.create_entry({
+            'user': self.user2,
+            'start_time': self.now - datetime.timedelta(hours=1),
+            'end_time': self.now
+        })
+
+        response = self.client.get(self.approve_url)
+        self.assertEquals(response.status_code, 403)
+
+        response = self.client.post(self.approve_url, {'do_action': 'Yes'})
+        self.assertEquals(response.status_code, 403)
+        self.assertNotEquals(entry.status, 'approved')
+        self.assertContains(response,
+            'Forbidden: You cannot approve this timesheet',
+            status_code=403
+        )
+
+    def test_verify_active_entry(self):
+        """
+        A user shouldnt be able to verify a timesheet if it contains
+        an active entry and should be redirect back to the ledger
+        """
+        self.login_as_admin()
+
+        entry1 = self.create_entry({
+            'user': self.user,
+            'start_time': self.now - datetime.timedelta(hours=5),
+            'end_time': self.now - datetime.timedelta(hours=4),
+            'status': 'unverified'
+        })
+        entry2 = self.create_entry({
+            'user': self.user,
+            'start_time': self.now - datetime.timedelta(hours=1),
+            'status': 'unverified'
+        })
+
+        response = self.client.get(self.verify_url, follow=True)
+        self.assertEquals(response.status_code, 200)
+
+        messages = response.context['messages']
+        msg = 'You cannot verify/approve this timesheet while the user {0} ' \
+            'has an active entry. Please have them close any active ' \
+            'entries.'.format(self.user.get_full_name())
+
+        self.assertEquals(messages._loaded_messages[0].message, msg)
+        self.assertEquals(entry1.status, 'unverified')
+        self.assertEquals(entry2.status, 'unverified')
+
+        response = self.client.post(self.verify_url, follow=True)
+        self.assertEquals(response.status_code, 200)
+        messages = response.context['messages']
+
+        self.assertEquals(messages._loaded_messages[0].message, msg)
+        self.assertEquals(entry1.status, 'unverified')
+        self.assertEquals(entry2.status, 'unverified')
+
     def testVerifyButton(self):
         response = self.client.get(self.sheet_url)
         self.assertNotContains(response, self.verify_url)
@@ -813,16 +1216,7 @@ class StatusTest(TimepieceDataTestCase):
         self.assertFalse(response.context['show_verify'])
 
     def testApproveButton(self):
-        edit_time_sheet = Permission.objects.get(codename='change_entry',
-            content_type__app_label='timepiece'
-        )
-        self.user2.user_permissions.add(edit_time_sheet)
-        view_time_sheet = Permission.objects.get(
-            codename=('view_entry_summary')
-        )
-        self.user2.user_permissions.add(view_time_sheet)
-        self.user2.save()
-        self.client.login(username='user2', password='abc')
+        self.login_as_admin()
         response = self.client.get(self.sheet_url)
         self.assertFalse(response.context['show_approve'])
         entry = self.create_entry(data={
@@ -856,43 +1250,103 @@ class StatusTest(TimepieceDataTestCase):
         self.assertEquals(entries[0].status, 'verified')
 
     def testApprovePage(self):
-        edit_time_sheet = Permission.objects.get(codename='change_entry',
-            content_type__app_label='timepiece'
-        )
-        self.user2.user_permissions.add(edit_time_sheet)
-        view_time_sheet = Permission.objects.get(
-            codename=('view_entry_summary')
-        )
-        self.user2.user_permissions.add(view_time_sheet)
-        self.user2.save()
-        self.client.login(username='user2', password='abc')
-
+        self.login_with_permission()
         entry = self.create_entry(data={
             'user': self.user,
             'start_time': datetime.datetime.now() - \
                 datetime.timedelta(hours=1),
             'end_time':  datetime.datetime.now(),
         })
-        response = self.client.post(self.approve_url, {'do_action': 'Yes'})
-        entries = self.user.timepiece_entries.all()
-        self.assertEquals(entries[0].status, 'unverified')
+
+        self.assertEquals(entry.status, 'unverified')
         entry.status = 'verified'
         entry.save()
 
         response = self.client.get(self.approve_url,)
-        entries = self.user.timepiece_entries.all()
-        self.assertEquals(entries[0].status, 'verified')
+        self.assertEquals(entry.status, 'verified')
 
         response = self.client.post(self.approve_url, {'do_action': 'Yes'})
-        self.assertEquals(entries[0].status, 'approved')
+        entry = timepiece.Entry.objects.get(pk=entry.pk)
+        self.assertEquals(entry.status, 'approved')
 
-    def testNotAllowedToAproveTimesheet(self):
+    def test_reject_user(self):
+        """A regular user should not be able to reject an entry"""
+        self.client.login(username='user', password='abc')
+
+        now = datetime.datetime.now()
+        entry = self.create_entry({
+            'user': self.user,
+            'start_time': now - datetime.timedelta(hours=1),
+            'end_time': now,
+            'status': 'verified'
+        })
+        url = self.get_reject_url(entry.pk)
+
+        response = self.client.post(url, {'Yes': 'yes'})
+        self.assertEquals(entry.status, 'verified')
+
+    def test_reject_other_user(self):
+        """
+        A regular user should not be able to reject
+        another users entry
+        """
+        self.client.login(username='user2', password='abc')
+
+        now = datetime.datetime.now()
+        entry = self.create_entry({
+            'user': self.user,
+            'start_time': now - datetime.timedelta(hours=1),
+            'end_time': now,
+            'status': 'verified'
+        })
+        url = self.get_reject_url(entry.pk)
+
+        response = self.client.post(url, {'Yes': 'yes'})
+        self.assertEquals(entry.status, 'verified')
+
+    def testRejectPage(self):
+        self.login_as_admin()
+        entry = self.create_entry(data={
+            'user': self.user,
+            'start_time': datetime.datetime.now() - \
+                datetime.timedelta(hours=1),
+            'end_time':  datetime.datetime.now(),
+        })
+        reject_url = self.get_reject_url(entry.id)
+
+        def check_entry_against_code(status, status_code):
+            entry.status = status
+            entry.save()
+            response = self.client.get(reject_url)
+            self.assertEqual(response.status_code, status_code)
+
+        check_entry_against_code('unverified', 302)
+        check_entry_against_code('invoiced', 302)
+        check_entry_against_code('approved', 200)
+        check_entry_against_code('verified', 200)
+        response = self.client.post(reject_url, {'Yes': 'yes'})
+        self.assertTrue(response.status_code, 302)
+        entry = timepiece.Entry.objects.get(user=self.user)
+        self.assertEqual(entry.status, 'unverified')
+
+    def testNotAllowedToRejectTimesheet(self):
+        entry = self.create_entry(data={
+            'user': self.user,
+            'start_time': datetime.datetime.now() - \
+                datetime.timedelta(hours=1),
+            'end_time':  datetime.datetime.now(),
+        })
+        reject_url = self.get_reject_url(entry.id)
+        response = self.client.get(reject_url)
+        self.assertTrue(response.status_code, 403)
+
+    def testNotAllowedToApproveTimesheet(self):
         response = self.client.get(self.approve_url,)
         self.assertTrue(response.status_code, 403)
 
     def testNotAllowedToVerifyTimesheet(self):
         self.client.login(username='user2', password='abc')
-        response = self.client.get(self.approve_url,)
+        response = self.client.get(self.verify_url,)
         self.assertTrue(response.status_code, 403)
 
 
@@ -908,36 +1362,35 @@ class TestTotals(TimepieceDataTestCase):
     def testGroupedTotals(self):
         self.client.login(username='user', password='abc')
         days = [
+                datetime.datetime(2010, 12, 20),
+                datetime.datetime(2010, 12, 27),
+                datetime.datetime(2010, 12, 28),
                 datetime.datetime(2011, 1, 3),
                 datetime.datetime(2011, 1, 4),
                 datetime.datetime(2011, 1, 10),
                 datetime.datetime(2011, 1, 16),
                 datetime.datetime(2011, 1, 17),
-                datetime.datetime(2011, 1, 18)
+                datetime.datetime(2011, 1, 18),
+                datetime.datetime(2011, 2, 2),
         ]
-        self.log_time(project=self.p1, start=days[0], delta=(1, 0))
-        self.log_time(project=self.p2, start=days[0], delta=(1, 0))
-        self.log_time(project=self.p4, start=days[0], delta=(1, 0))
-        self.log_time(project=self.p1, start=days[1], delta=(1, 0))
-        self.log_time(project=self.p3, start=days[1], delta=(1, 0))
-        self.log_time(project=self.p4, start=days[1], delta=(1, 0))
-        self.log_time(project=self.p1, start=days[2], delta=(1, 0))
-        self.log_time(project=self.p2, start=days[2], delta=(1, 0))
-        self.log_time(project=self.p4, start=days[2], delta=(1, 0))
-        self.log_time(project=self.p1, start=days[3], delta=(1, 0))
-        self.log_time(project=self.p3, start=days[3], delta=(1, 0))
-        self.log_time(project=self.p4, start=days[3], delta=(1, 0))
-        self.log_time(project=self.p1, start=days[4], delta=(1, 0))
-        self.log_time(project=self.p2, start=days[4], delta=(1, 0))
-        self.log_time(project=self.p4, start=days[4], delta=(1, 0))
-        self.log_time(project=self.p1, start=days[5], delta=(1, 0))
-        self.log_time(project=self.p3, start=days[5], delta=(1, 0))
-        self.log_time(project=self.p4, start=days[5], delta=(1, 0))
-        entries = timepiece.Entry.objects.all()
+        # Each week has two days of entries, except 12-20, and 2-2 but these
+        # are excluded in the timespan queryset
+        for day in days:
+            self.log_time(project=self.p1, start=day, delta=(1, 0))
+            self.log_time(project=self.p4, start=day, delta=(1, 0))
+            if random.choice([True, False]):
+                self.log_time(project=self.p2, start=day, delta=(1, 0))
+            else:
+                self.log_time(project=self.p3, start=day, delta=(1, 0))
+        from_date = utils.get_month_start(datetime.datetime(2011, 1, 19))
+        to_date = from_date + relativedelta(months=1)
+        first_week = utils.get_week_start(from_date)
+        entries = timepiece.Entry.objects.timespan(first_week, to_date=to_date)
         grouped_totals = utils.grouped_totals(entries)
         for week, week_totals, days in grouped_totals:
             #Jan. 3rd is a monday. Each week should be on a monday
-            self.assertEqual(week.day % 7, 3)
+            if week.month == 1:
+                self.assertEqual(week.day % 7, 3)
             self.assertEqual(week_totals['billable'], 4)
             self.assertEqual(week_totals['non_billable'], 2)
             self.assertEqual(week_totals['total'], 6)
@@ -959,3 +1412,99 @@ class TestTotals(TimepieceDataTestCase):
                     if project == self.p4:
                         self.assertEqual(totals['billable'], 1)
                         self.assertEqual(totals['total'], 1)
+
+
+class HourlySummaryTest(TimepieceDataTestCase):
+    def setUp(self):
+        super(HourlySummaryTest, self).setUp()
+        self.now = datetime.datetime.now()
+        self.month = self.now.replace(day=1)
+        self.url = reverse('view_person_time_sheet', args=(self.user.pk,))
+        self.client.login(username='user', password='abc')
+
+    def create_month_entries(self):
+        """Create four entries, one for each week of the month"""
+        self.create_entry({
+            'user': self.user,
+            'start_time': self.month,
+            'end_time': self.month + datetime.timedelta(hours=1)
+        })
+        self.create_entry({
+            'user': self.user,
+            'start_time': self.month + datetime.timedelta(weeks=1),
+            'end_time': self.month + datetime.timedelta(weeks=1, hours=1)
+        })
+        self.create_entry({
+            'user': self.user,
+            'start_time': self.month + datetime.timedelta(weeks=2),
+            'end_time': self.month + datetime.timedelta(weeks=2, hours=1)
+        })
+        self.create_entry({
+            'user': self.user,
+            'start_time': self.month + datetime.timedelta(weeks=3),
+            'end_time': self.month + datetime.timedelta(weeks=3, hours=1)
+        })
+
+    def test_start_of_week(self):
+        """Test that the entries start being labled on the first week, ISO"""
+        self.create_month_entries()
+
+        response = self.client.get(self.url)
+        self.assertEquals(response.status_code, 200)
+
+        start_date = utils.get_week_start(self.month)
+        msg = 'Week of {0}'.format(start_date.strftime('%m/%d/%Y'))
+        self.assertContains(response, msg)
+
+    def test_contains_only_current_entries(self):
+        """
+        Only entries from the current month should be displayed
+        using default data from create_month_entries()
+        """
+        self.create_month_entries()
+        old_entry = self.create_entry({
+            'user': self.user,
+            'start_time': self.month - datetime.timedelta(days=1, hours=1),
+            'end_time': self.month - datetime.timedelta(days=1)
+        })
+
+        response = self.client.get(self.url)
+        self.assertEquals(response.status_code, 200)
+        self.assertFalse(old_entry in response.context['entries'])
+
+    def test_single_entry_in_week(self):
+        """
+        When there is a single entry at the end of an ISO week,
+        the view should show the entries from that entire week
+        even though they belong in the previous month.
+
+        This occurs in April 2012, so we are using that month
+        as the basis for out test case
+        """
+        april = datetime.datetime(month=4, day=1, year=2012)
+        march = datetime.datetime(month=3, day=26, year=2012)
+        self.create_entry({
+            'user': self.user,
+            'start_time': april,
+            'end_time': april + datetime.timedelta(hours=1)
+        })
+        self.create_entry({
+            'user': self.user,
+            'start_time': april + datetime.timedelta(weeks=1),
+            'end_time': april + datetime.timedelta(weeks=1, hours=1)
+        })
+        self.create_entry({
+            'user': self.user,
+            'start_time': march,
+            'end_time': march + datetime.timedelta(hours=1)
+        })
+
+        response = self.client.get(self.url + '?{0}'.format(
+            urllib.urlencode({'year': 2012, 'month': 4})
+        ))
+        self.assertEquals(response.status_code, 200)
+        # entries context object is a ValuesQuerySet
+        entries = timepiece.Entry.objects \
+            .timespan(april, span='month') \
+            .date_trunc('month', True)
+        self.assertEquals(list(entries), list(response.context['entries']))
