@@ -14,6 +14,8 @@ urlpatterns = patterns('',
     url(r'^add/$', views.create_edit_entry, name='timepiece-add'),
     url(r'^update/(?P<entry_id>\d+)/$', views.create_edit_entry,
         name='timepiece-update'),
+    url(r'^reject/(?P<entry_id>\d+)/$', views.reject_entry,
+        name='timepiece-reject-entry'),
     url(r'^delete/(?P<entry_id>\d+)/$', views.delete_entry,
         name='timepiece-delete'),
     url(r'^search/$', views.quick_search, name='quick_search'),
@@ -82,6 +84,23 @@ urlpatterns = patterns('',
         name='edit_project_relationship',
     ),
 
+
+    url(
+        r'^project/(?P<pk>\d+)/delete/$',
+        views.DeleteProjectView.as_view(),
+        name='delete_project',
+    ),
+    url(
+        r'^business/(?P<pk>\d+)/delete/$',
+        views.DeleteBusinessView.as_view(),
+        name='delete_business',
+    ),
+    url(
+        r'^person/(?P<pk>\d+)/delete/$',
+        views.DeletePersonView.as_view(),
+        name='delete_person',
+    ),
+
     ### time sheets ###
 
     # Reports
