@@ -25,10 +25,10 @@ from django.conf import settings
 from django.utils.datastructures import SortedDict
 from django.views.decorators.csrf import csrf_exempt
 from django.views.generic.base import TemplateView
-from django.views.generic import UpdateView, ListView, DetailView
+from django.views.generic import UpdateView, ListView, DetailView, View
 from django.utils.decorators import method_decorator
 
-from timepiece.utils import render_with
+from timepiece.utils import render_with, reverse_lazy
 
 from timepiece import models as timepiece
 from timepiece import utils
@@ -42,7 +42,11 @@ def quick_search(request):
         form = timepiece_forms.QuickSearchForm(request.GET)
         if form.is_valid():
             return HttpResponseRedirect(form.save())
-    raise Http404
+    return render_to_response('timepiece/search_results.html', {
+            'form': form,
+        },
+        context_instance=RequestContext(request)
+    )
 
 
 class CSVMixin(object):
@@ -67,6 +71,9 @@ class CSVMixin(object):
 @login_required
 @render_with('timepiece/time-sheet/dashboard.html')
 def view_entries(request):
+    view_entries = False
+    if request.user.has_perm('timepiece.can_clock_in'):
+        view_entries = True
     week_start = utils.get_week_start()
     entries = timepiece.Entry.objects.select_related(
         'project__business',
@@ -85,6 +92,7 @@ def view_entries(request):
     activity_entries = entries.values(
         'billable',
     ).annotate(sum=Sum('hours')).order_by('-sum')
+    current_total = entries.aggregate(sum=Sum('hours'))['sum']
     others_active_entries = timepiece.Entry.objects.filter(
         end_time__isnull=True,
     ).exclude(
@@ -122,8 +130,10 @@ def view_entries(request):
         'schedule': schedule,
         'project_entries': project_entries,
         'activity_entries': activity_entries,
+        'current_total': current_total,
         'others_active_entries': others_active_entries,
         'my_active_entries': my_active_entries,
+        'view_entries': view_entries,
     }
     return context
 
@@ -132,39 +142,27 @@ def view_entries(request):
 @transaction.commit_on_success
 def clock_in(request):
     """For clocking the user into a project"""
-    if request.POST:
-        form = timepiece_forms.ClockInForm(request.POST, user=request.user)
-        if form.is_valid():
-            entry = form.save()
-            #check that the user is not currently logged into another project.
-            #if so, clock them out of all others.
-            my_active_entries = timepiece.Entry.no_join.select_related(
-                'project__business',
-            ).filter(
-                user=request.user,
-                end_time__isnull=True,
-            ).exclude(
-                id=entry.id
-            )
-            #clock_out all open entries one second before the last
-            for sec_bump, active_entry in enumerate(my_active_entries):
-                active_entry.unpause()
-                active_entry.end_time = entry.start_time - \
-                    datetime.timedelta(seconds=sec_bump + 1)
-                active_entry.save()
-
-            request.user.message_set.create(
-                message='You have clocked into %s' % entry.project)
-            return HttpResponseRedirect(reverse('timepiece-entries'))
-        else:
-            request.user.message_set.create(
-                message='Please correct the errors below.')
-    else:
-        initial = dict([(k, request.GET[k]) for k in request.GET.keys()])
-        form = timepiece_forms.ClockInForm(user=request.user, initial=initial)
-    return render_to_response(
-        'timepiece/time-sheet/entry/clock_in.html',
-        {'form': form},
+    active_entry = timepiece.Entry.no_join.filter(user=request.user,
+                                                  end_time__isnull=True)
+    # Should never happen, but just in case.
+    if len(active_entry) > 1:
+        err_msg = 'You have more than one active entry and must clock out ' \
+                  'of these entries before clocking into another.'
+        messages.error(request, err_msg)
+        return redirect('timepiece-entries')
+    active_entry = active_entry[0] if active_entry else None
+    initial = dict([(k, v) for k, v in request.GET.items()])
+    form = timepiece_forms.ClockInForm(request.POST or None, initial=initial,
+                                       user=request.user, active=active_entry)
+    if form.is_valid():
+        entry = form.save()
+        message = 'You have clocked into %s' % entry.project
+        messages.info(request, message)
+        return HttpResponseRedirect(reverse('timepiece-entries'))
+    return render_to_response('timepiece/time-sheet/entry/clock_in.html', {
+            'form': form,
+            'active': active_entry,
+        },
         context_instance=RequestContext(request),
     )
 
@@ -181,11 +179,12 @@ def clock_out(request, entry_id):
         form = timepiece_forms.ClockOutForm(request.POST, instance=entry)
         if form.is_valid():
             entry = form.save()
-            request.user.message_set.create(message="You've been clocked out.")
+            message = "You've been clocked out."
+            messages.info(request, message)
             return HttpResponseRedirect(reverse('timepiece-entries'))
         else:
-            request.user.message_set.create(
-                message='Please correct the errors below.')
+            message = 'Please correct the errors below.'
+            messages.error(request, message)
     else:
         form = timepiece_forms.ClockOutForm(instance=entry)
     context = {
@@ -215,8 +214,8 @@ def toggle_paused(request, entry_id):
                                   end_time__isnull=True)
     except:
         # create an error message for the user
-        request.user.message_set.create(
-            message='The entry could not be paused.  Please try again.')
+        message = 'The entry could not be paused.  Please try again.'
+        messages.error(request, message)
     else:
         # toggle the paused state
         entry.toggle_paused()
@@ -243,7 +242,7 @@ def toggle_paused(request, entry_id):
         message = 'The log entry has been %s. %s' % (action, duration)
 
         # create a message that can be displayed to the user
-        request.user.message_set.create(message=message)
+        messages.info(request, message)
 
     # redirect to the log entry list
     return HttpResponseRedirect(reverse('timepiece-entries'))
@@ -279,12 +278,11 @@ def create_edit_entry(request, entry_id=None):
                 message = 'The entry has been updated successfully.'
             else:
                 message = 'The entry has been created successfully.'
-            request.user.message_set.create(message=message)
+            messages.info(request, message)
             return HttpResponseRedirect(reverse('timepiece-entries'))
         else:
-            request.user.message_set.create(
-                message='Please fix the errors below.',
-            )
+            message = 'Please fix the errors below.'
+            messages.error(request, message)
     else:
         initial = dict([(k, request.GET[k]) for k in request.GET.keys()])
         form = timepiece_forms.AddUpdateEntryForm(
@@ -299,6 +297,39 @@ def create_edit_entry(request, entry_id=None):
     }
 
 
+@permission_required('timepiece.view_payroll_summary')
+def reject_entry(request, entry_id):
+    """
+    Admins can reject an entry that has been verified or approved but not
+    invoiced to set its status to 'unverified' for the user to fix.
+    """
+    user = request.user
+    return_url = request.REQUEST.get('next', reverse('timepiece-entries'))
+    try:
+        entry = timepiece.Entry.no_join.get(pk=entry_id)
+    except:
+        message = 'No such log entry.'
+        messages.error(request, message)
+        return redirect(return_url)
+
+    if entry.status == 'unverified' or entry.status == 'invoiced':
+        msg_text = 'This entry is unverified or is already invoiced'
+        messages.error(request, msg_text)
+        return redirect(return_url)
+
+    if request.POST.get('Yes'):
+        entry.status = 'unverified'
+        entry.save()
+        msg_text = "The entry's status was set to unverified"
+        messages.info(request, msg_text)
+        return redirect(return_url)
+    return render_to_response('timepiece/time-sheet/entry/reject_entry.html', {
+                                  'entry': entry,
+                                  'next': request.REQUEST.get('next'),
+                              },
+                              context_instance=RequestContext(request))
+
+
 @permission_required('timepiece.delete_entry')
 def delete_entry(request, entry_id):
     """
@@ -314,18 +345,20 @@ def delete_entry(request, entry_id):
                                   user=request.user)
     except:
         # entry does not exist
-        request.user.message_set.create(message='No such log entry.')
+        message = 'No such log entry.'
+        messages.info(request, message)
         return HttpResponseRedirect(reverse('timepiece-entries'))
 
     if request.method == 'POST':
         key = request.POST.get('key', None)
         if key and key == entry.delete_key:
             entry.delete()
-            request.user.message_set.create(message='Entry deleted.')
+            message = 'Entry deleted.'
+            messages.info(request, message)
             return HttpResponseRedirect(reverse('timepiece-entries'))
         else:
-            request.user.message_set.create(
-                message='You are not authorized to delete this entry!')
+            message = 'You are not authorized to delete this entry!'
+            messages.error(request, message)
 
     return render_to_response('timepiece/time-sheet/entry/delete_entry.html',
                               {'entry': entry},
@@ -474,18 +507,56 @@ def view_person_time_sheet(request, user_id):
     if not (request.user.has_perm('timepiece.view_entry_summary') or \
         user.pk == request.user.pk):
         return HttpResponseForbidden('Forbidden')
-    year_month_form = timepiece_forms.YearMonthForm(request.GET or None)
-    if request.GET and year_month_form.is_valid():
-        from_date, to_date = year_month_form.save()
-    else:
-        from_date = utils.get_month_start(datetime.datetime.today()).date()
-        to_date = from_date + relativedelta(months=1)
-    entries_qs = timepiece.Entry.objects
-    entries_qs = entries_qs.timespan(from_date, span='month').filter(user=user)
+    today_reset = datetime.datetime.today()
+    today_reset = today_reset.replace(hour=0, minute=0, second=0, \
+        microsecond=0)
+    from_date = utils.get_month_start(today_reset)
+    to_date = from_date + relativedelta(months=1)
+    can_view_summary = request.user and \
+        request.user.has_perm('timepiece.view_entry_summary')
+    form = timepiece_forms.UserYearMonthForm if can_view_summary else \
+        timepiece_forms.YearMonthForm
+    year_month_form = form(request.GET or None)
+    if year_month_form.is_valid():
+        if can_view_summary:
+            from_date, to_date, form_user = year_month_form.save()
+            is_update = request.GET.get('yearmonth', None)
+            if form_user and is_update:
+                url = reverse('view_person_time_sheet', args=(form_user.pk,))
+                # Do not use request.GET in urlencode in case it has the
+                # yearmonth parameter (redirect loop otherwise)
+                request_data = {
+                    'month': from_date.month,
+                    'year': from_date.year,
+                    'user': form_user.pk
+                }
+                url += '?{0}'.format(urllib.urlencode(request_data))
+                return HttpResponseRedirect(url)
+        else:
+            from_date, to_date = year_month_form.save()
+    entries_qs = timepiece.Entry.objects.filter(user=user)
+    month_qs = entries_qs.timespan(from_date, span='month')
+    month_entries = month_qs.date_trunc('month', True)
+    # For grouped entries, back date up to the start of the week.
+    first_week = utils.get_week_start(from_date)
+    month_week = first_week + datetime.timedelta(weeks=1)
+    grouped_qs = entries_qs.timespan(first_week, to_date=to_date)
+    intersection = grouped_qs.filter(start_time__lt=month_week,
+        start_time__gte=from_date)
+    # If the month of the first week starts in the previous
+    # month and we dont have entries in that previous ISO
+    # week, then update the first week to start at the first
+    # of the actual month
+    if not intersection and first_week.month < from_date.month:
+        grouped_qs = entries_qs.timespan(from_date, to_date=to_date)
+    grouped_totals = utils.grouped_totals(grouped_qs) if month_entries else ''
+    project_entries = month_qs.order_by().values(
+        'project__name').annotate(sum=Sum('hours')).order_by('-sum')
+    summary = timepiece.Entry.summary(user, from_date, to_date)
     show_approve = show_verify = False
     if request.user.has_perm('timepiece.change_entry') or \
         user == request.user:
-        statuses = list(entries_qs.values_list('status', flat=True))
+        statuses = list(month_qs.values_list('status', flat=True))
         total_statuses = len(statuses)
         unverified_count = statuses.count('unverified')
         verified_count = statuses.count('verified')
@@ -494,18 +565,13 @@ def view_person_time_sheet(request, user_id):
     if request.user.has_perm('timepiece.change_entry'):
         show_approve = verified_count + approved_count == total_statuses \
         and verified_count > 0 and total_statuses != 0
-    month_entries = entries_qs.date_trunc('month', True)
-    grouped_totals = utils.grouped_totals(entries_qs) if month_entries else ''
-    project_entries = entries_qs.order_by().values(
-        'project__name').annotate(sum=Sum('hours')).order_by('-sum')
-    summary = timepiece.Entry.summary(user, from_date, to_date)
     context = {
         'year_month_form': year_month_form,
         'from_date': from_date,
         'to_date': to_date - datetime.timedelta(days=1),
         'show_verify': show_verify,
         'show_approve': show_approve,
-        'user': user,
+        'timesheet_user': user,
         'entries': month_entries,
         'grouped_totals': grouped_totals,
         'project_entries': project_entries,
@@ -518,9 +584,18 @@ def view_person_time_sheet(request, user_id):
 @login_required
 def change_person_time_sheet(request, action, user_id, from_date):
     user = get_object_or_404(User, pk=user_id)
-    admin_verify = request.user.has_perm('timepiece.change_entry')
-    if not admin_verify and user != request.user:
-        return HttpResponseForbidden('Forbidden')
+    admin_verify = request.user.has_perm('timepiece.view_entry_summary')
+    perm = True
+
+    if not admin_verify and action == 'verify' and user != request.user:
+        perm = False
+    if not admin_verify and action == 'approve':
+        perm = False
+
+    if not perm:
+        return HttpResponseForbidden('Forbidden: You cannot {0} this ' \
+            'timesheet'.format(action))
+
     try:
         from_date = datetime.datetime.strptime(from_date, '%Y-%m-%d')
     except (ValueError, OverflowError):
@@ -529,17 +604,30 @@ def change_person_time_sheet(request, action, user_id, from_date):
     entries = timepiece.Entry.no_join.filter(user=user_id,
                                              end_time__gte=from_date,
                                              end_time__lt=to_date)
+    active_entries = timepiece.Entry.no_join.filter(
+        user=user_id,
+        start_time__lt=to_date,
+        end_time=None,
+        status='unverified'
+    )
     filter_status = {
         'verify': 'unverified',
         'approve': 'verified',
     }
     entries = entries.filter(status=filter_status[action])
+
     return_url = reverse('view_person_time_sheet', kwargs={'user_id': user_id})
     return_url += '?%s' % urllib.urlencode({
         'year': from_date.year,
         'month': from_date.month,
     })
-    if request.POST and request.POST.get('do_action', 'No') == 'Yes':
+    if active_entries:
+        msg = 'You cannot verify/approve this timesheet while the user {0} ' \
+            'has an active entry. Please have them close any active ' \
+            'entries.'.format(user.get_full_name())
+        messages.error(request, msg)
+        return redirect(return_url)
+    if request.POST.get('do_action') == 'Yes':
         update_status = {
             'verify': 'verified',
             'approve': 'approved',
@@ -548,13 +636,18 @@ def change_person_time_sheet(request, action, user_id, from_date):
         messages.info(request,
             'Your entries have been %s' % update_status[action])
         return redirect(return_url)
+    hours = entries.all().aggregate(s=Sum('hours'))['s']
+    if not hours:
+        msg = 'You cannot verify/approve a timesheet with no hours'
+        messages.error(request, msg)
+        return redirect(return_url)
     context = {
         'action': action,
-        'user': user,
+        'timesheet_user': user,
         'from_date': from_date,
         'to_date': to_date - datetime.timedelta(days=1),
         'return_url': return_url,
-        'hours': entries.all().aggregate(s=Sum('hours'))['s'],
+        'hours': hours,
     }
     return render_to_response('timepiece/time-sheet/people/change_status.html',
         context, context_instance=RequestContext(request))
@@ -563,7 +656,7 @@ def change_person_time_sheet(request, action, user_id, from_date):
 @login_required
 @transaction.commit_on_success
 def confirm_invoice_project(request, project_id, to_date, from_date=None):
-    if not request.user.has_perm('timepiece.change_entry'):
+    if not request.user.has_perm('timepiece.generate_project_invoice'):
         return HttpResponseForbidden('Forbidden')
     try:
         to_date = datetime.datetime.strptime(to_date, '%Y-%m-%d')
@@ -828,6 +921,7 @@ def view_business(request, business):
     return context
 
 
+@permission_required('timepiece.add_business')
 @render_with('timepiece/business/create_edit.html')
 def create_edit_business(request, business=None):
     if business:
@@ -942,13 +1036,12 @@ def create_edit_person(request, person_id=None):
 @permission_required('timepiece.view_project')
 @render_with('timepiece/project/list.html')
 def list_projects(request):
-    form = timepiece_forms.SearchForm(request.GET)
-    if form.is_valid() and 'search' in request.GET:
-        search = form.cleaned_data['search']
+    form = timepiece_forms.ProjectSearchForm(request.GET)
+    if form.is_valid():
+        search, status = form.save()
         projects = timepiece.Project.objects.filter(
-            Q(name__icontains=search) |
-            Q(description__icontains=search)
-        )
+            Q(name__icontains=search) | Q(description__icontains=search))
+        projects = projects.filter(status=status) if status else projects
         if projects.count() == 1:
             url_kwargs = {
                 'project_id': projects[0].id,
@@ -1182,7 +1275,7 @@ def edit_settings(request):
     return {'profile_form': profile_form, 'user_form': user_form}
 
 
-@permission_required('timepiece.view_payroll_summary')
+@permission_required('timepiece.view_entry_summary')
 @render_with('timepiece/time-sheet/reports/hourly.html')
 @utils.date_filter
 def hourly_report(request, date_form, from_date, to_date, status, activity):
@@ -1267,3 +1360,61 @@ class ContractList(ListView):
     @method_decorator(permission_required('timepiece.add_project_contract'))
     def dispatch(self, *args, **kwargs):
         return super(ContractList, self).dispatch(*args, **kwargs)
+
+
+class DeleteView(TemplateView):
+    model = None
+    url_name = None
+    permissions = None
+    form_class = timepiece_forms.DeleteForm
+    template_name = 'timepiece/delete_object.html'
+
+    def dispatch(self, request, *args, **kwargs):
+        for permission in self.permissions:
+            if not request.user.has_perm(permission):
+                messages.info(request, 'You do not have permission to access that')
+                return HttpResponseRedirect(reverse_lazy('timepiece-entries'))
+        return super(DeleteView, self).dispatch(request, *args, **kwargs)
+
+    def post(self, request, *args, **kwargs):
+        instance = self.get_queryset(**kwargs)
+        form = self.form_class(request.POST, instance=instance)
+        msg = '{0} could not be successfully deleted'.format(instance)
+
+        if form.is_valid():
+            if form.save():
+                msg = '{0} was successfully deleted'.format(instance)
+
+        messages.info(request, msg)
+        return HttpResponseRedirect(reverse_lazy(self.url_name))
+
+    def get(self, request, *args, **kwargs):
+        context = self.get_context_data(*args, **kwargs)
+        return self.render_to_response(context)
+
+    def get_queryset(self, **kwargs):
+        pk = kwargs.get('pk', None)
+        return get_object_or_404(self.model, pk=pk)
+
+    def get_context_data(self, *args, **kwargs):
+        context = super(DeleteView, self).get_context_data(*args, **kwargs)
+        context['object'] = self.get_queryset(**kwargs)
+        return context
+
+
+class DeletePersonView(DeleteView):
+    model = User
+    url_name = 'list_people'
+    permissions = ('auth.add_user', 'auth.change_user',)
+
+
+class DeleteBusinessView(DeleteView):
+    model = timepiece.Business
+    url_name = 'list_businesses'
+    permissions = ('timepiece.add_business',)
+
+
+class DeleteProjectView(DeleteView):
+    model = timepiece.Project
+    url_name = 'list_projects'
+    permissions = ('timepiece.add_project', 'timepiece.change_project',)
